use std::rc::Rc;

use super::{Command, types::Content};

pub fn commands() -> Vec<Rc<Command>> {
    let mut available_commands: Vec<Rc<Command>> = Vec::new();

    available_commands.push(
        Rc::new(Command::new(
        "debug".to_string(),
        "Prints the args and flags".to_string(),
        vec![
            Rc::new(super::Flag::new("long".to_string(), "l".to_string(), "A long flag".to_string())),
            Rc::new(super::Flag::new("short".to_string(), "s".to_string(), "A short flag".to_string())),
        ],
        |_console, args, flags| {
            let mut output = String::from("Flags");
            for flag in flags {
                output.push_str(&format!("\n\tLong: {}\n\tShort: {}\n\tDescription: {}", flag.long, flag.short, flag.description));
            }

            output.push_str("\nArgs:");
            for arg in args {
                output.push_str(&format!("\n\t{}", arg));
            }

            Ok(Some(output))
        },
    )));

    available_commands.push(
        Rc::new(Command::new(
        "echo".to_string(),
        "Prints the args".to_string(),
        Vec::new(),
        |_console, args, _flags| {
            let mut output = String::new();
            for arg in args {
                output.push_str(&format!("{} ", arg));
            }

            Ok(Some(output))
        },
    )));

    available_commands.push(
        Rc::new(Command::new(
        "pwd".to_string(),
        "Prints the current working directory".to_string(),
        Vec::new(),
        |console, args, _flags| {
            if args.len() > 0 {
                return Err("pwd does not take any arguments");
            }

            Ok(Some(console.get_current_dir().borrow().get_path()))
        },
    )));

    available_commands.push(
        Rc::new(Command::new(
            "cd".to_string(),
            "Changes the current working directory".to_string(),
            Vec::new(),
            |console, args, _flags| {
                if args.len() != 1 {
                    return Err("Only the path must be specified!");
                }

                match console.cd(args[0].clone()) {
                    Ok(_) => Ok(None),
                    Err(e) => Err(e),
                }
            },
    )));

    available_commands.push(
        Rc::new(Command::new(
            "ls".to_string(),
            "Lists the contents of the current directory".to_string(),
            vec![
                Rc::new(super::Flag::new("list".to_string(), "l".to_string(), "Print contents in a list".to_string())),
            ],
            |console, args, flags| {
                if args.len() > 0 {
                    return Err("ls does not take any arguments");
                }

                // TODO
                // Virtual cd to read the contents there

                let seperator = match flags.iter().find(|f| f.long == "list" ) {
                    Some(_) => "\n",
                    None => "\t",
                };

                let mut output = String::new();
                let current_dir = console.get_current_dir();
                for dir in current_dir.borrow().get_subdirs() {
                    output.push_str(&format!("{}{}", dir.borrow().get_name(), seperator));
                }
                for file in current_dir.borrow().get_files() {
                    output.push_str(&format!("{}{}", file.borrow().get_name(), seperator));
                }

                Ok(Some(output))
            },
    )));

    available_commands.push(
        Rc::new(Command::new(
            "mkdir".to_string(),
            "Creates a new directory".to_string(),
            Vec::new(),
            |console, args, _flags| {
                if args.len() != 1 {
                    return Err("Only the name must be specified!");
                }

                // TODO
                // virtual_cd
                // Get the directory name from the path

                let current_dir = console.get_current_dir();
                let result = match current_dir.borrow_mut().mkdir(args[0].clone()) {
                    Ok(_) => Ok(None),
                    Err(e) => Err(e),
                };

                result
            },
    )));

    available_commands.push(
        Rc::new(Command::new(
            "touch".to_string(),
            "Creates a new file".to_string(),
            Vec::new(),
            |console, args, _flags| {
                if args.len() != 1 {
                    return Err("Only the name must be specified!");
                }

                // TODO
                // virtual_cd
                // Get the directory name from the path

                let current_dir = console.get_current_dir();
                let result = match current_dir.borrow_mut().touch(args[0].clone()) {
                    Ok(_) => Ok(None),
                    Err(e) => Err(e),
                };

                result
            },
    )));

    available_commands.push(
        Rc::new(Command::new(
            "cat".to_string(),
            "Prints the contents of a file".to_string(),
            Vec::new(),
            |console, args, _flags| {
                if args.len() != 1 {
                    return Err("Only the name must be specified!");
                }

                // TODO
                // virtual_cd
                // Get the directory name from the path

                let current_dir = console.get_current_dir();
                let file = match current_dir.borrow().get_file(args[0].clone()) {
                    Some(file) => file,
                    None => return Err("File not found"),
                };

                let content = file.borrow().read();

                Ok(Some(content))
            },
    )));

    available_commands.push(
        Rc::new(Command::new(
            "rm".to_string(),
            "Removes a file or directory".to_string(),
            Vec::new(),
            |console, args, _flags| {
                if args.len() != 1 {
                    return Err("Only the name must be specified!");
                }

                // TODO
                // virtual_cd
                // Get the directory name from the path

                let current_dir = console.get_current_dir();
                let result = match current_dir.borrow_mut().remove_file(args[0].clone()) {
                    Ok(_) => Ok(None),
                    Err(e) => Err(e),
                };

                result
            },
    )));

    // write into file
    available_commands.push(
        Rc::new(Command::new(
            "write".to_string(),
            "Writes into a file".to_string(),
            vec![
                Rc::new(super::Flag::new("append".to_string(), "a".to_string(), "Append to the file".to_string())),
            ],
            |console, args, flags| {
                // TODO
                // virtual_cd
                // Get the directory name from the path

                let current_dir = console.get_current_dir();
                let file = match current_dir.borrow().get_file(args[0].clone()) {
                    Some(file) => file,
                    None => return Err("File not found"),
                };

                if flags.iter().find(|f| f.long == "append" ).is_some() {
                    file.borrow_mut().append(args[1].clone());
                    return Ok(None);
                } else {
                    file.borrow_mut().write(args[1].clone());
                    Ok(None)
                }
                
            },
    )));

    available_commands
}